<?php

declare(strict_types=1);

namespace Drupal\registration_limit\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\registration_limit\Mapper\LastLoginMapper;
use Drupal\user\UserDataInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Service to work with client ips.
 */
class ClientIpService {

  /**
   * Construct new instance.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\user\UserDataInterface $userData
   *   The user data service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\registration_limit\Mapper\LastLoginMapper $lastLoginMapper
   *   The last loggin mapper.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected UserDataInterface $userData,
    protected RequestStack $requestStack,
    protected LastLoginMapper $lastLoginMapper,
    protected LoggerInterface $logger,
  ) {}

  /**
   * Saves current request ip for given user.
   *
   * @param string|int $uid
   *   The user id.
   */
  public function saveRequestIpForUser(string|int $uid): void {
    $request = $this->requestStack->getCurrentRequest();
    $ip = $request->getClientIp();

    $this->lastLoginMapper->saveLastLoginData($uid, $ip);
  }

  /**
   * Checks recent logins and sessions for given IP address.
   *
   * @param string $ip
   *   The IP address to check for.
   */
  public function ipRecentlyUsed(string $ip): bool {
    // Check whitelist.
    if ($this->isIpWhitelisted($ip)) {
      $this->logger->info('IP Whitelisted @ip', [
        '@ip' => $ip,
      ]);

      return FALSE;
    }

    return $this->lastLoginMapper->ipRecentlyUsedForLogin($ip) ||
      $this->lastLoginMapper->ipRecentlyUsedInSession($ip);
  }

  /**
   * Checks if given ip address in whitelisted.
   *
   * @param string $ip
   *   The IP address to check.
   */
  protected function isIpWhitelisted(string $ip): bool {
    $ipWhitelist = \Drupal::config('registration_limit.settings')->get('ip_whitelist') ?? '';

    $ipAddresses = explode(PHP_EOL, $ipWhitelist);

    foreach ($ipAddresses as $ipAddress) {
      if (trim($ipAddress) === $ip) {
        return TRUE;
      }
    }

    return FALSE;
  }

}
